;
; Creation par AC le 25/11/2005 a Paris
;
; Purpose: ajout facile d'une legende composee d'un titre,
; et d'une serie de valeurs, en associant a chaque valeur
; son type de trace (bout de ligne, avec couleur, linestyle, psym et thick)
;
; -----------------------------------------
; Historique des modifications:
;
; AC le 14/12/05:
; -- modification du calcul de l'echelle
; -- verification dans AC_LEGENDE_DEMO avec un cadre
; -- ajour de l'option "title"
;
; AC le 15/12/2005 (au LAL, bat 208):
; -- ajout de l'option BOX
; -- gestion plus fine des _Extra (thick, charsize, charthick)
;
; -----------------------------------------
; Advices:
; Des parametres peuvent etre geres soit par un chgt externe de !p,
; ou bien par un passage par parametre "extra"
; -----------------------------------------
; Usage:
; -- see AC_LEGENDE_DEMO ci-bas.
; -----------------------------------------
;
pro AC_LEGENDE, positions, messages, title=title, colors=colors, $
                linestyle=linestyle, box=box, _extra=_extra, $
                verbose=verbose, debug=debug, test=test, help=help
;
mess=AC_SETMESSAGES('AC_LEGENDE')
;
if (N_PARAMS() lt 2) then begin
    print, mess.fatal+'missing arguments'
    help=1
endif
;
if KEYWORD_SET(help) then begin
    print, 'pro AC_LEGENDE, positions, messages, title=title, colors=colors, $'
    print, '                linestyle=linestyle, box=box, _extra=_extra, $'
    print, '                verbose=verbose, debug=debug, test=test, help=help'
    return
endif
;
xmin=positions[0]
xmax=positions[1]
xmid=(xmin+xmax)/2.
xdx=(xmax-xmin)/20.
;
ymin=positions[2]
ymax=positions[3]
;
; Gestion de !P et des _extra
; si le parametre Thick est passe, on mmodifie aussi
; la taille des caracteres !
;
sysvar_modified=0
if (SIZE(_extra))(0) GT 0 then begin
    sysvar_modified=1
    copy_p=!p
    if IS_FIELD_IN_STRUCT(_extra, 'thick') then !p.thick=_extra.thick
    if IS_FIELD_IN_STRUCT(_extra, 'charthick') then !p.charthick=_extra.charthick
    if IS_FIELD_IN_STRUCT(_extra, 'charsize') then !p.charsize=_extra.charsize
endif
;
; il n'y a qu'un titre (a ce jour)
; il faut recalculer le y_max
;
nbp=N_ELEMENTS(messages)
;
if KEYWORD_SET(title) then begin
    dy=(ymax-ymin)/FLOAT(nbp+1)
    ymax=ymin+dy/2.+dy*nbp
    XYOUTS, xmid, ymax, title, ALIGNMENT=0.5
    if KEYWORD_SET(box) then begin
        ;; trait sous le titre
        y_subtitle=ymax-dy/2.
        PLOTS, [xmin,xmax], [y_subtitle,y_subtitle]
    endif
endif else begin
    dy=(ymax-ymin)/FLOAT(nbp)
endelse
;
for j=0, nbp-1 do begin
    ;; pour garder l'ordre actuel (du plus petit au plus grand)
    y=ymin+dy/2.+dy*(nbp-1-j)
    PLOTS, [xmin,xmid], [y,y], color=colors(j), $
      line=linestyle(j), _extra=_extra
    XYOUTS, xmid+xdx, y, messages(j)
endfor
;
; eventuel trace de la boite encadrant la legende
;
if KEYWORD_SET(box) then begin
    ;; on revient aux "vrais" bornes en Y
    ymin=positions[2]
    ymax=positions[3]
    ;;
    PLOTS, [xmin,xmax], [ymin,ymin] ;; trait bas horizontal
    PLOTS, [xmin,xmax], [ymax,ymax] ;; trait haut horizontal
    PLOTS, [xmin,xmin], [ymin,ymax] ;; premier trait vertical
    PLOTS, [xmax,xmax], [ymin,ymax] ;; second trait vertical
endif
;
; remise en etat de !P
if (sysvar_modified EQ 1) then !P=copy_p
;
if KEYWORD_SET(test) then STOP
;
end
;
; ------------------------------------------
;
pro AC_LEGENDE_DEMO, test=test
;
messages=['red','green','blue','yellow']
colors=[!col.r, !col.g, !col.b, !col.y]
lines=[1,2,3,4]
;
plot, findgen(100)
;
AC_LEGENDE, 100.*[0.5,0.9,0.5,0.9], messages, /box, $
  colors=colors, lines=lines, thick=1.5, title='coucou'
;
; 2 fois plus de traits, une fois avec titre, une fois sans titre
;
messages=[messages,messages]
colors=[colors,colors]
lines=[lines,lines]
;
xmin=15
xmax=30
ymin=5
ymax=50
;
pref=!p
!p.thick=3.
!p.charsize=2.
AC_LEGENDE, [xmin,xmax,ymin,ymax], messages, /box, $
  colors=colors, lines=lines, charsize=0.75
!p=pref
;
xmin=40
xmax=50
ymin=5
ymax=50
;
AC_LEGENDE, [xmin,xmax,ymin,ymax], messages, /box, $
  colors=colors, lines=lines, thick=1.5, title='coucou'
;
if KEYWORD_SET(test) then STOP
;
end
