;
; AC, Tucson, juin 2001 ?
;
; Trace une ou des lignes, horizontales ou verticales,
; a des niveaux donnes, en fonction des echelles du
; trace deja existant (n'ecrit pas au dela des limites)
;
; NB: le mot_clef _extra permet de transmettre des parametres
; a la fonction PLOT d'IDL ... (couleur, epaisseur et styles des traits ...)
;
; -----------------------------------
; Historique des modifications:
; AC le 27/07/2004:
; -- Ajout de l'option Vertical
; -- Mise a niveau de la doc
;
; AC le 19/05/2005:
; -- la procedure est renonnee de AC_LIGNES en AC_LINES
; -- la procedure est deplace dans l'arbo Systeme et mise a niveau
; -- les mots-clef MINI, MAXI et PAS qui ne sont plus utilises
;    suite a une evolution du code sont supprimes
;
; AC le 15/09/2005:
; -- ceci ne marche pas lorsqu'on est en log/lin ou log/log
;    parce qu'alors les CRANGE sont l'exposant ...
;
; -----------------------------------
;
pro AC_LINES, liste_niveaux, vertical=vertical, $
              xlog=xlog, ylog=ylog, _extra=e, $
              help=help, test=test
;
mess=AC_SETMESSAGES("AC_LINES")
;
if (N_ELEMENTS(liste_niveaux) LT 1) then begin
    print, mess.fatal+"missing mandatory list : [liste_niveaux]"
    help=1
endif
;
if KEYWORD_SET(help) then begin
    print, 'pro AC_LINES, liste_niveaux, vertical=vertical, $'
    print, '              xlog=xlog, ylog=ylog, _extra=e, $
    print, '              _extra=e, help=help, test=test'
    print, ''
    print, 'This procedure overplot Lines at given level(s)'
    print, ''
    print, 'CALLING SEQUENCE : AC_LINES, findgen(10), /vertical, /ylog'
    return
endif
;
; 2 cas: lignes verticales ou horizontales
;
if KEYWORD_SET(vertical) then begin
    ;;
    ;; ici, on trace des lignes VERTICALES
    ;;
    y_range=[!y.crange(0),!y.crange(1)]
    if KEYWORD_SET(ylog) then y_range=10^y_range
    ;;
    ;; premier cas : on founit une liste de niveaux
    ;;
    nb_niveaux=N_ELEMENTS(liste_niveaux)
    if (nb_niveaux GE 1) then begin
        for i=0, (nb_niveaux-1) do begin
            niveau=liste_niveaux(i)
            PLOTS, [niveau,niveau], y_range, _extra=e
        endfor
    endif
endif else begin
    ;;
    ;; ici, on trace des lignes HORIZONTALES
    ;;
    x_range=[!x.crange(0),!x.crange(1)]
    if KEYWORD_SET(xlog) then x_range=10^x_range
    ;;
    ;; premier cas : on founit une liste de niveaux
    ;;
    nb_niveaux=N_ELEMENTS(liste_niveaux)
    if (nb_niveaux GE 1) then begin
        for i=0, (nb_niveaux-1) do begin
            niveau=liste_niveaux(i)
            PLOTS, x_range, [niveau,niveau], _extra=e
        endfor
    endif
endelse
;
if KEYWORD_SET(test) then STOP
;
end
