;
; AC le 09/11/2005
;
; Ce creneau est normalisee entre 0 et 1
;
; --------------------------------
; Liste des modifications:
;
; AC le 12/12/05: ajout du mot-clef ISHIFT pour indiquer
; l'indice de decalage ou le creneau DOIT passer a zero
; Attention, si les periodes ne sont pas entieres, avec ISHIFT
; on aura des problemes de "bout" non complet recentre ...
;
; --------------------------------
; Exemples d'usage:
;
; a=CRENEAUX(ish=128,/dis, /xsty,psym=2,fre=2)
; Ce creneau est decale d'une demi-periode.
;
; --------------------------------
;
function CRENEAUX, nbp=nbp, frequency=frequency, duration=duration, $
                   ishift=ishift, _extra=_extra, $
                   help=help, test=test, verbose=verbose, display=display
;
mess=AC_SETMESSAGES('CRENEAUX')
;
if KEYWORD_SET(help) then begin
    print, 'function CRENEAUX, nbp=nbp, frequency=frequency, duration=duration, $'
    print, '                   ishift=ishift, _extra=_extra, $'   
    print, '                   help=help, test=test, verbose=verbose, display=display'
    print, ' '
    print, 'CALLING SEQUENCE : signal=CRENEAUX(nbp=1024, duration=1.)'
    return, -1
endif
;
if N_ELEMENTS(nbp) EQ 0 then nbp=512
if N_ELEMENTS(frequency) EQ 0 then frequency=10. ; En Hertz
if N_ELEMENTS(duration) EQ 0 then duration=1.  ; En secondes
if N_ELEMENTS(ishift) EQ 1 then begin
    if ((ishift LT 0) OR (ishift GE nbp)) then begin
        print, mess.warning+'wrong range for ISHIFT, reset to Zero !'
        ishift=0
    endif
endif
;
if KEYWORD_SET(verbose) then begin
    print, mess.info+'Total number of Nbp: ....', nbp
    print, mess.info+'Frequency (Hertz) : .....', frequency
    print, mess.info+'Duration (seconds): .....', duration
    if KEYWORD_SET(ishift) then print, mess.info+'IShift (indices): .....', ishift
endif    
;
time_raw = INDGEN(nbp)/FLOAT(nbp)*frequency*duration
signal = ROUND(time_raw mod 1.)
time=time_raw/frequency
;
; application de l'eventuel decallage
;
if KEYWORD_SET(ishift) then signal=SHIFT(signal,ishift)
;
; eventuel affichage
;
if KEYWORD_SET(display) then begin
    WINDOW, /free
    PLOT, time, signal, xtitle='Signal Duration (sec)', $
      _extra=_extra, yrange=[-0.1, 1.1], /ystyle
    if KEYWORD_SET(ishift) then AC_LINES, time(ishift), /vertical, color=!col.r
    AC_DELAIS
    WDELETE
endif
;
if KEYWORD_SET(test) then STOP
;
return, signal
;
end
