FUNCTION index2ij,index,sizes, help=help

;+
; NAME:
;       index2ij
; CALLING SEQUENCE:
;       coor=index2ij(ind,sizes)
; PURPOSE:
;       returns the pixel postion of index ind in an array
; INPUTS:
;       index = the index for which coordinates are needed (value or 1D)
;	   sizes = Array dimensions
; OPTIONAL INPUT:
;	   None
; OUTPUTS:
;	   coordinates corresponding to the index
; PROCEDURE AND SUBROUTINE USED
;       
; SIDE EFFECTS:
; MODIFICATION HISTORY:
;       written by Jean-Philippe Bernard 09-92
;	   modified JPB 11-Sep-92 for 2D arrays
;	   modified JPB to allow complete indexes to be handled
;       modified JPB 21-June-94 for large sets of indexes
;       debugged AC 31/03/2003 for pb with single value
; EXAMPLE:
;       a=fltarr(12,15,17)
;       a(10,10,5)=1 & a(11,5,16)=1
;       ind=where(a NE 0,count)
;       ij=index2ij(ind,[12,15,17])
;       help,ij
;       print,ij
;       ind2=ij2index(ij,[12,15,17])
;       print,ind,ind2
;-

;on_error,2

mess=AC_SETMESSAGES('INDEX2IJ')

if (N_PARAMS(0) ne 2) then begin
    print, mess.fatal+'Calling sequence: coor=INDEX2IJ(index,sizes)'
    help=1
endif
if KEYWORD_SET(help) then begin
    print,'function INDEX2IJ,index, sizes, help=help'
    print, ''
    print,'index can be 1D vector or single value'
    return, -1
endif
;
taille=N_ELEMENTS(sizes)
si=SIZE(index)
;
; AC, le 31/03/2003
;
if (si(0) EQ 0) then si=SIZE([index])    
;
siind=si(0)

res=LONARR(si(1),taille)
FOR i=0l,si(1)-1 DO BEGIN
  ind_val=index(i)
  FOR tt=taille,2,-1 DO BEGIN
    size_prod=1.
    FOR j=0,tt-2 DO size_prod=size_prod*sizes(j)
    res(i,tt-1)=FIX(ind_val/size_prod)
    ind_val=ind_val-size_prod*res(i,tt-1)
  ENDFOR
;  res(i,1)=fix(ind_val/sizes(0))
  res(i,0)=fix(ind_val)
ENDFOR  
;
return,res
;
end
