;
; AC, le 01/04/2005 a Paris
;
; Les parametres Bolo sont stockes dans un fichier ASCII
; Il faut les relire pour les mettre dans une structure.
;
; Malheureusement, la procedure "READ_ASCII" ne sait pas lire
; les fichiers melangeant des nombres et du texte
; Mais on s'en sert pour recuperer le nombre de lignes utiles
;
;----------------------------------------------
; Conventions pour les fichiers de donnees Bolo
;
; Les lignes de commentaires commencent par un "#"
; Les commentaires sont au debut
; La premiere ligne utile est factice, elle sert pour les relectures
; par tableur (OOcalc, MS-Excel) au format CSV
; Les lignes utiles comptent exactement 10 ";"
; Les lignes utiles peuvent contenir, a la fin, un commentaire prefixe
; par un caractere "#"
;----------------------------------------------
; Liste de modifications:
;
; AC le 03/06/2005 
; -correction d'un bug lie aux versions d'IDL lorsqu'on lit des
; structures anonymes
; (pb trouve grace a CREATE_HTML_GALLERY.pro et READF_CSV.pro)
;
;----------------------------------------------
;
function READ_BOLO_CSV, filename=filename, path=path, $
                        prefixe=prefixe, suffixe=suffixe, $
                        verbose=verbose, debug=debug, test=test, help=help
;
mess=AC_SETMESSAGES('LIST_BOLO_READFILE')
;
; si le mot-clef /help est active, on retourne l'aide et on sort
;
if KEYWORD_SET(help) then begin
    print, 'function LIST_BOLO_READFILE, filename=filename, path=path, $'
    print, '            verbose=verbose,debug=debug, test=test, help=help'
    print, ''
    print, 'This function reads an ASCII file containing'
    print, 'a list of Bolometers parameters'
    print, 'The output is stored in a list of structure'
    print, ''
    print, 'CALLING SEQUENCE : list_bolo_struct=LIST_BOLO_READFILE()'
    return, -1
endif
;
if N_ELEMENTS(filename) EQ 0 then begin
    filename='boloCQM_v1.csv'
    print, mess.info+'Default Bolo INFO FileName used:', filename
endif
;
; AC 21/05/2005 : we do not manage fully the PATH now
;
if N_ELEMENTS(path) EQ 0 then begin
    path='BoloData'+!path_sep
    print, mess.info+'Default Path: ./', path
endif
;
full_filename=path+filename
;
; On s'assure que le Fichier est bien present
;
liste_file=AC_FINDFILE(full_filename, count=nb_file)
if (nb_file LT 1) then begin
    print, mess.fatal+'Bolo INFO FileName <<'+filename+'>>,'
    print, mess.fatal+'in PATH <<'+path+'>> does not exist,'
    print, mess.fatal+'Please re-run this function with another filename.'
    return, -1
endif
;
; Conventions dans le fichier ASCII :
;   - les separateurs sont des ","
;   - les commentaires sont apres des "#"
;   - la premiere colonne contient le Nom du bolo
;   - la 2ieme colonne contient des info sur les unites (K, W, ...)
;
a=READ_ASCII(full_filename, com='#', del=';');, RECORD_START=)
a=READ_ASCII_FILE(filename=filename, path=path, $
                  prefixe=prefixe, suffixe=suffixe)
;
delim=','
nb_fields=AC_STR_COUNT(a, delim)

clefs=['"R*"','"Tg"','"b"',"G0"]
nb_clefs=N_ELEMENTS(clefs)
for ii=0, nb_clefs-1 do begin
    
endfor

;
; on retire la premiere ligne qui contient les noms de chaque champ
;
tab_lines=tab_lines(1:*)
nb_bolo=nb_bolo-1
;
if KEYWORD_SET(debug) then stop
if KEYWORD_SET(verbose) then begin
    print, mess.info+'Total number of bolo read in file', N_ELEMENTS(tab_lines)
endif
;
; template pour la struture bolo
; (attention aux convertions forcees de type (si Int dans la
; declaration, reste ensuite a Int))
;
bolo={name   :'template', $
      Freq   : 111., $       ;; en GHz
      flag   : 1, $          ;; un flag qui dit si le bolo est OK
      Polar  : 0, $          ;; boolean: 0 si flux, 1 si polar
      Rinf   : 100., $       ;; anciennement "r0"
      Ttrans : 10., $         ;; anciennement "Tg"
      Alpha  : 0.5, $        ;; anciennement "m"
      Gbol   : 100.e-12, $   ;; anciennement "g_s0"
      Beta   : 1., $
      c0     : 1.e-12, $
      dcapa  : 1., $
      i_polar: 1.e-10, $
      Tref   : 0.1}   ;; T_bath (Kelvin)
;
bolo_list=REPLICATE(bolo,nb_bolo)
for i=0,nb_bolo-1 do begin
    splited_line=STRSPLIT(tab_lines(i), ';',/extract) 
    ;;
    ;; on recopie le String et on retire les blancs
    bolo_list(i).Name=StrCompress(splited_line(0),/remove_all)
    ;;
    bolo_list(i).Freq=FLOAT(splited_line(1))
    if (FIX(splited_line(2)) EQ 1) then begin
        bolo_list(i).Polar=FIX(splited_line(3))
        bolo_list(i).Ttrans=FLOAT(splited_line(4))
        bolo_list(i).Rinf=FLOAT(splited_line(5))
        bolo_list(i).Beta=FLOAT(splited_line(6))
        bolo_list(i).Gbol=FLOAT(splited_line(7))*1e-12  ; conversion ...
        bolo_list(i).c0=FLOAT(splited_line(8))
        bolo_list(i).dcapa=FLOAT(splited_line(9))
    endif else begin
        bolo_list(i).flag=0
    endelse
endfor
;
; on filtre la liste pour ne garder que les bolo
; dont les champs sont complets (toutes les info sont dispo)
;
bolo_list=LIST_BOLO_EXTRACT(bolo_list, /flag)
;
nb_good_bolo=N_ELEMENTS(bolo_list)
;
if KEYWORD_SET(verbose) then begin
    print, mess.info+'Total number of GOOD bolo : .....', nb_good_bolo
endif
;
if (nb_good_bolo GT 0) then begin
    print, mess.info, nb_good_bolo, ' Good Bolo are now available.'
endif else begin
    print, mess.warning+'NO good bolo are now available.'
    print, mess.warning+'Please check PATH and Filename.'
endelse
;
if KEYWORD_SET(debug) OR KEYWORD_SET(test) then stop
;
return, bolo_list
;
end
