;
; AC, debut novembre 2005
;
; A partir d'un fichier de reponse courant/frequence de modulation
; optique/signal electrique, calcul des niveaux de back/signal et
; fit des parametres dynamiques d'un bolo
;
; -------------------------------------------
; -------------------------------------------
;
pro FIT_BOLO_SPEED, speed_name=speed_name, bolo_name=bolo_name, $
                    sub_data=sub_data, T_bath=T_bath, $
                    maxiter=maxiter, postscript=postscript, $
                    test=test, debug=debug, help=help, verbose=verbose
;
mess=AC_SETMESSAGES('FIT_BOLO_SPEED')
;
if KEYWORD_SET(help) then begin
    print, 'pro FIT_BOLO_SPEED, speed_name=speed_name, bolo_name=bolo_name, $'
    print, '         maxiter=maxiter, postscript=postscript, $'   
    print, '         test=test, debug=debug, help=help, verbose=verbose'
    return
endif
;
;if NOT(KEYWORD_SET(speed_name)) then speed_name='Speed_100.csv'
;if NOT(KEYWORD_SET(bolo_name)) then bolo_name='100-1'
;
; lecture des donnees
print, '<<',speed_name,'>>'
print, '<<',bolo_name,'>>'
;stop
;
OK=AC_SELECTFILE(filename=speed_name, pref='Speed_', suf='.csv', path='Data/')
if (OK EQ 1) then begin
    SPEED_DATA_READFILE, filename=speed_name, data=data, $
      frequencies=frequencies, currents=currents
    print, mess.info+'data file: '+speed_name+'was succesfully read'
endif else return
;
; lecture du bolometre
;
bolo=-1
if KEYWORD_SET(bolo_name) then begin
    bolo_list=LIST_BOLO_READFILE(/verbose)
    bolo=LIST_BOLO_EXTRACT(bolo_list, name=bolo_name,/verbose)
    if (N_TAGS(bolo) LE 0) then begin
        print, mess.warning+'Bolo name not found, switch to interactive mode'
        bolo=-1
    endif
endif
if (N_TAGS(bolo) LE 0) then begin
    bolo=LIST_BOLO_SELECT()
    if (N_TAGS(bolo) LE 0) then begin
        print, mess.fatal+'No selected bolo !'
        return
    endif
endif
;
print, mess.info, 'selected bolo is: ', bolo.name
if KEYWORD_SET(verbose) then print, mess.info, 'bolo parameters are:', bolo 
;
; gestion des parametres libres
;
flux_back=1e-12
flux_signal=1.e-13
;
if (bolo.freq EQ 100.) then begin
    flux_back  =8.30e-13
    flux_signal=2.e-14
endif
if (bolo.freq EQ 143.) then begin
    flux_back  =2.2e-12
    flux_signal=1.e-14
endif
if (bolo.freq EQ 217.) then begin
    flux_back  =1.80e-12
    flux_signal=0.17e-12
endif
if (bolo.freq EQ 353.) then begin
    flux_back  =2.30e-12
    flux_signal=0.64e-12
endif
;
; recopie des donnees brutes
;
init_data=data
;
; on ne prend qu'un petit sous ensemble des donnees initiales
; (on recopie d'abord)
;
full_currents=currents
full_freq=frequencies
full_data=data
;
full=1
;
if KEYWORD_SET(sub_data) then begin
    full=0
    sub_cur=[1,2,4,5,6,8]
    currents=currents(sub_cur)
    data=data(sub_cur,*)
    sub_freq=INDGEN(17)*2+5
    data=data(*,sub_freq)
    frequencies=frequencies(sub_freq)
endif
;
bolo.Dcapa=1.5
;
start_parms=[flux_back,flux_back+flux_signal, $
            bolo.freq, bolo.flag, bolo.polar, bolo.Rinf, bolo.Ttrans, $
            bolo.alpha, bolo.Gbol, bolo.beta, bolo.c0, bolo.Dcapa, bolo.Tref]
;
; A quelle T_bath travaille-t-on ?
;
if (N_ELEMENTS(T_bath) EQ 1) then start_parms(12)=T_bath
;
nb_parms=N_ELEMENTS(start_parms)
; par defaut, les parametre sont fixes
parinfo = replicate({value:0., fixed:1, limited:[0,0], limits:[0.,0.]}, nb_parms)
parinfo(*).value=start_parms
;
; on ne joue que sur Back, signal, Dcapa
;
parinfo([0,1,10,11]).fixed=[0,0,0,0]
;
parinfo(0).limited(*)=[1,1]
parinfo(0).limits(*)=[1e-15,1e-10]
;
parinfo(1).limited(*)=[1,1]
parinfo(1).limits(*)=[1e-15,1e-11]
;
parinfo(10).limited(*)=[1,1]
parinfo(10).limits(*)=[1.0e-13, 1.0e-12]
;
parinfo(11).limited(*)=[1,1]
parinfo(11).limits(*)=[.25,2.]
;
if KEYWORD_SET(debug) then STOP
if (N_ELEMENTS(maxiter) EQ 1) then begin
    if (maxiter LE 0) then maxiter=15
endif else maxiter=15
;
; le fit en lui-meme
;
parms=MPFIT2DFUN("FUNC_BOLO_SPEED", frequencies, currents, data, $
                 WEIGHTS=1, parinfo=parinfo, maxiter=maxiter)
;
r1=FUNC_BOLO_SPEED(frequencies, currents, start_parms)
r2=FUNC_BOLO_SPEED(frequencies, currents, parms)
;
line=INDGEN(N_ELEMENTS(full_currents)) mod 4
;
; superposition des donnees, du modele avec les valeurs initiales (rouge)
; et du modele valeurs fittees (vert) sur un nombre limite de points
;
SPEED_DATA_PLOT, data, frequencies, currents, line=line, psym=-2, title=speed_name
SPEED_DATA_PLOT, r1, frequencies, currents, /over, col=!col.r, line=line, psym=-2
SPEED_DATA_PLOT, r2, frequencies, currents, /over, col=!col.g, line=line, psym=-2
;
; la meme chose sur "tout"
;
r2full=FUNC_BOLO_SPEED(full_freq, full_currents, parms)
;
if KEYWORD_SET(postscript) then begin
    set_plot, 'ps'
    device, file='speed_bolo'+bolo.name+'.ps', /color, bit=8
endif
;
SPEED_DATA_PLOT, full_data, full_freq, full_currents, $
  line=line, psym=-2, title=speed_name
SPEED_DATA_PLOT, r2full, full_freq, full_currents, $
  /over, col=!col.g, line=line, psym=-2
;
if KEYWORD_SET(postscript) then begin
    device, /close
    set_plot, 'x'
endif
;
fit_save_file='fit_parms_'+bolo.name+'_i'+STRING(maxiter)+'_f'+STRING(full)+'.xdr'
fit_save_file=STRCOMPRESS(fit_save_file,/remove_all)
;
SAVE, file=fit_save_file, maxiter, frequencies, currents, $
  start_parms, parms, parinfo, bolo, speed_name, full
;
print, mess.info+'Fit parms file was saved: ', fit_save_file
;
if KEYWORD_SET(test) then STOP
;
end
;
