;
; Creation par AC le 19/03/2004 a Meudon
;
; But: ce programme est un test de correction "readout par readout"
;      de la reponse du bolometre: on fournit les parametres du bolo,
;      la temperature mesuree et on cherche a retrouver le flux
;      optique incident.
;
; Pour le moment, on garde les parametres des bolo car ceci pourrait
; etre utile pour tester les consequences d'une erreur (meconnaissance
; ou autre) sur un des parametres ...
;
; N.B.: Ce programme est base sur "Equa_Temp_Bolo"
;
; Historique des modifications:
;
; AC le 08/04/2004:
; - le programme est renomme de CORRECT_TEMP_BOLO en BOLO_INVERT_MODEL
;
; Dependances (Ce programme necessite):
;      AC_SETMESSAGES 
;
function BOLO_INVERT_MODEL, TempIn, $   ;; temperature d'entree (array)
                            nbp=nbp, delta_t=delta_t, $
                            current=current, $ ;; polarization current (array)
                            bolo=bolo, $            ;; Structure "bolo"
                            r0=r0, Tg=Tg, m=m, $ ;; parametres de P Joule
                            g_s0=g_s0, beta=beta, $ ;; parametres de P fuite
                            c0=c0, dcapa=dcapa, $ ;; parametres de P interne Bolo
                            display=display, bavard=bavard, $
                            debug=debug, test=test, help=help
;
mess=AC_SETMESSAGES('BOLO_INVERT_MODEL')
;
;if (N_PARAMS() lt 1) then begin
;	print, mess.fatal+'missing arguments'
;	help=1
;endif
;
if KEYWORD_SET(help) then begin
    print, ' Not up to date'
    ;;
    print, 'Default values are provided for each parameter ! (/bavard to print them)'
    print, 'P Joule: params (r0, Tg, m)'
    print, 'P bolo : params (c0, dcapa)'
    print, 'P fuite: params (g_s0, beta)'
    return, -1
endif
;
if (N_ELEMENTS(delta_t) NE 1) then delta_t=1e-4
if (delta_t LE 0.) then begin
    print, mess.fatal+'"delta_t" must be positive'
    return, -1
endif
;
; description de la Resistance (P Joule)
;
; valeur indiquee par M. Piat
; (Attention, notre definition est differente de celle dans Chanin/Torre 84)
if (N_ELEMENTS(Tg) NE 1) then Tg=16.54 ;en K
; Chanin Torre 1984 (p412b): (m=0.25, 0.5 ou 1)
if (N_ELEMENTS(m) NE 1) then m=.5
; valeur donnee par Sudiwala
if (N_ELEMENTS(r0) NE 1) then r0=120.48 ;en ohm
;
; description de la P fuite
;
; en W/K pour T=Tbain
if (N_ELEMENTS(g_s0) NE 1) then g_s0=60.*1e-12
;
; Chanin Torre 1984 : beta ~ 3 (p 413b)
; pour le cas planck, on a 1.2 a 1.5
if (N_ELEMENTS(beta) NE 1) then beta=1.5
;
; description de la capacite calorifique bolo
;
; "c0" is the capacity for temp==temp0 (0.1K)
if (N_ELEMENTS(c0) NE 1) then c0=0.8e-12 ; en J/K
; Chanin Torre 1984 (p418b)
; pour le cas planck, on a dcapa==2.
if (N_ELEMENTS(dcapa) NE 1) then dcapa=2.1
;
; si un "bolo" est passe, on a chaque champ rempli
; et on ecrase tout ce qui peut etre mis par param ...
;
; AC, modif des noms des variable le 03/03/04, d'apres XML IMO
;
if (N_ELEMENTS(bolo) GT 0) then begin
    r0=bolo.Rinf
    Tg=bolo.Ttrans
    m=bolo.Alpha
    g_s0=bolo.Gbol
    beta=bolo.Beta
    c0=bolo.c0
    dcapa=bolo.dcapa
    ;; ATTENTION: ceci veut dire que "current" est a priori tjrs
    ;; active par ici puisque le champ "bolo.i_polar" existe tjrs !
    ;; le 04/03/04: Je rajoute un petit test de warning !
    if (N_ELEMENTS(current) GT 0) then begin
        print, mess.warning+"Indeed, current is set by <<bolo.i_polar>>"
    endif
    current=bolo.i_polar
endif
;
double_precision=1
if KEYWORD_SET(double_precision) then begin
    r0=double(r0)
    Tg=Double(Tg)
    g_s0=Double(g_s0)
    c0=Double(c0)
    current=Double(current)
endif
;
if KEYWORD_SET(bavard) then begin
    print, mess.info+'P resistance (Tg, m, r0):', Tg, m, r0
    print, mess.info+'P interne Bolo (c0, dcapa):', c0, dcapa
    print, mess.info+'P fuite (g_s0, beta):', g_s0, beta
endif
;
; Nb de Points a calculer
;
nbp=N_ELEMENTS(TempIn)
if (nbp LE 1) then begin
    print, mess.warning+'Strange size of <<TempIn>> array'
    STOP
endif
;
; ---------------------------------------------------------------------
; Le courant
; si on ne fournit qu'une seule valeur, on le transforme en tableau
; il n'y a pas de probleme de signe puisque qu'on se sert de son carre
;
nbp_current=N_ELEMENTS(current)
if (nbp_current LE 0) then begin
    current=1e-9                ; en nA
    tab_i=REPLICATE(current,nbp)
endif
if (nbp_current EQ 1) then tab_i=REPLICATE(current,nbp)
if (nbp_current GT 1) then begin
    if (nbp_current NE nbp) then begin
        print, mess.fatal+" nbp_current <> nbp ! ", nbp_current, nbp
        return, -1
    endif
endif
;
; en sortie de ce "case" on a necessairement un "tableau" "tab_i"
;
r0i2=r0*tab_i*tab_i
;
; ---------------------------------------------------------------------
; creation des tableaux de reference
;
Temp0=DOUBLE(0.1)
OptSignal=DBLARR(nbp)
;
; AC, test Double Precision le 18/02/2004
; t=Delta_t*findgen(nbp)
t=Delta_t*Dindgen(nbp)
;
; ---------------------------------------------------------------------
;
; le calcul lui-meme, simple x(n+1)=x(n)+delta_t*Function(x(n), params)
;
;debug=1
if KEYWORD_SET(debug) then begin
    Pj_tab=Dblarr(nbp)
    C_tab=Dblarr(nbp)
    Pf_tab=Dblarr(nbp)
endif
;
for i=1L, (nbp-1L) do begin
    TempCur=TempIn(i-1)
    Pj=r0i2(i)*EXP((Tg/TempCur)^m)
    C=c0*(TempCur/Temp0)^dcapa
    Pf=g_s0*Temp0/(1.+beta)*((TempCur/Temp0)^(beta+1.)-1.)
    ;;
    OptSignal(i)=(TempIn(i)-TempCur)*c/Delta_t+Pf-Pj
    ;;
    if KEYWORD_SET(debug) then begin
        Pj_tab(i)=Pj
        C_tab(i)=C
        Pf_tab(i)=Pf
    endif
endfor
;
; ATTENTION: comme on ne peut pas calculer le 1er point, on y met
; la valeur mediane (ceci afin d'avoir OptSignal-Input-Back == noise )
;
OptSignal(0)=MEDIAN(OptSignal)
;
if KEYWORD_SET(display) then begin
    !P.multi=[0,0,2]
    WINDOW, xsize=900, ysize=900
    PLOT, t, TempIn, title='Temperature variation'
    PLOT, t, OptSignal, title='Estimated incoming optical flux'
    !p.multi=0
endif
;
; on recopie l'original ...
; Rappel: ci-dessous, signal=signal_ref+background ...
;
if (N_ELEMENTS(nbpSignal) GT 0) then signal=signal_ref
;
if KEYWORD_SET(debug) then begin
    stop
    ;; HELP
endif
;
if KEYWORD_SET(test) then STOP
;
return, OptSignal
;
end
;
