;
; AC, le 03/05/2005
; Purpose: This function computes in a Time Serie ("signal")
; the Time Constant of a upward or downward step
;
; It is assumed that the time serie :
;   -(1) contains one and only one step
;   -(2) is monotonic
; in order to simplify the code. (Absolutely no gestion of pb and errors)
;
; d'apres la formule magique de Michel Piat
;
; Liste des modificiations:
; AC le 26/09/2005: 
; -- 3 calculs differents (63%, 10/90 et 87%) d'apres
; http://perso.wanadoo.fr/f5zv/RADIO/RM/RM23/RM23B/RM23B23.html
;
function AC_TIMECONSTANTE, signal, delta_time=delta_time, $
                           test=test, display=display
;
; si le pas temporel n'est pas fourni, il est implicitement
; egal a 1
;
if (N_ELEMENTS(delta_time) EQ 0) then delta_time=1.
;
signal_min=MIN(signal)
signal_max=MAX(signal)
;
scale=signal_max-signal_min
;
val_at10=MIN(ABS(signal-(signal_min+0.1*scale)),i10)
val_at90=MIN(ABS(signal-(signal_max-0.1*scale)),i90)
;
time_constante=(i90-i10)/2.22*delta_time
;
; AC le 26/09/05 pour JML : doute sur l'efficacite de la formule de MPiat
;
i0=MAX(WHERE(signal EQ signal_min))
val_at63=MIN(ABS(signal-(signal_max-0.37*scale)),i63)
time63=(i63-i0)*delta_time
val_at87=MIN(ABS(signal-(signal_max-0.13*scale)),i87)
time87=(i87-i0)*delta_time/2.
;
print, '3 ways: ', time_constante, time63, time87
;
if KEYWORD_SET(display) then begin
    plot, signal, /ynozer
    oplot, [i10, i10], !y.crange, col=!col.r
    oplot, [i90, i90], !y.crange, col=!col.r
endif
;
if KEYWORD_SET(test) then STOP
;
return, time_constante
end
