;
; AC 26/09/2005
;
; This program is a superseeding of previous pro TEST_TIME_CONST
;
;------------------------------
; Historique des modifications:
;
; le 26/09/2005: AC
; - Creation from TEST_TIME_CONST: change from procedure to function
;
function CALCUL_TIME_CONST, bolo=bolo, amp=amp, back=back, $
                            nbp=nbp, up=up, down=down, theory=theory, $
                            T_bath=T_bath, current=current, display=display, $
                            help=help, test=test, verbose=verbose
;
mess=AC_SETMESSAGES('CALCUL_TIME_CONST')
;
if KEYWORD_SET(help) then begin
    print, 'function CALCUL_TIME_CONST, bolo=bolo, amp=amp, back=back, $'
    print, '                        nbp=nbp, T_bath=T_bath, current=current, $'
    print, '                        help=help, test=test, verbose=verbose'
    print, ''
    print, 'parameter AMP  : amplitude of the optical step'
    print, 'parameter BACK : amplitude of the optical background'
    print, ' '
    print, 'CALLING SEQUENCE: tau=CALCUL_TIME_CONST(T_bath=.130)'
endif
;
; on ferme toutes les fenetres actives afin que
; la succession des traces soit claire !
;
AC_RESET
;
; selection interactive d'un bolo
;
if (N_TAGS(bolo) LE 0) then begin
    print, mess.fatal+'No selected bolo !'
    return, -1
endif
;
if (N_ELEMENTS(nbp) EQ 0) then nbp=1e4
;
delta_t=1e-4
;
; we have a default source level, this can be changed through Keyword AMP=
;
if (N_ELEMENTS(amp) EQ 0) then amp=2.7e-13
;
; creation du signal incident
;
op_up=CREATE_OPT_SIGNAL(delta_t=delta_t, nbp=nbp, /step, amp=amp, /center)
op_down=CREATE_OPT_SIGNAL(delta_t=delta_t, nbp=nbp, /step, amp=-amp, /center)
print, mess.info+'Don''t care : the NEGATIVE value is OK here !'
;
; we add a background, this can be changed through Keyword BACK=
;
if (N_ELEMENTS(back) EQ 0) then back=3.8e-13
op_up=op_up+back
op_down=op_down+back
;
; on previent quand meme si les valeurs sont aberrantes
; MAIS on ne gere pas d'arret !
;
out=OUT_OF_RANGE(amp, min=1e-15,max=1e-10,'Amplitude')
out=OUT_OF_RANGE(back, min=1e-15,max=1e-10,'Background')
;
; ----------------------------------------
; Quelle temperature pour T_Bath
; (ici, on ne gere que le cas ou T_Bath est constant !)
;
if (N_ELEMENTS(T_bath) EQ 0) then T_bath=0.100
out=OUT_OF_RANGE(T_bath, min=0.050, max=0.300, 'T_bath')
if (T_bath LE 0.0) then begin
    print, mess.fatal+'T_bath must be positive'
    STOP
endif
;
; Quel courant ? (NB: on se moque qu'il soit negative, car ^2)
; (ici, on ne gere que le cas ou le courant est constant !)
;
if (N_ELEMENTS(current) EQ 0) then current=1e-9
out=OUT_OF_RANGE(current, min=1e-12, max=1e-7,'Current')
;
if KEYWORD_SET(verbose) then begin
    print, mess.info+'Background level (W):', back
    print, mess.info+'Step level (W):', amp
    print, mess.info+'T_bath (K):', T_bath
    print, mess.info+'Current (A):', current
endif
;
; ----------------------------------------
;
t_up=BOLO_DIRECT_MODEL(op_up, bolo=bolo, t_bath=t_bath, current=current)
t_down=BOLO_DIRECT_MODEL(op_down, bolo=bolo, t_bath=t_bath, current=current)
;
;plot, t, /ynoze
;
xr=1000
;
if KEYWORD_SET(display) then begin
    !p.multi=[0,0,2]
    plot, t_up(nbp/2-xr/10:nbp/2+2*xr), /ynoze
    plot, t_down(nbp/2-xr/10:nbp/2+2*xr), /ynoze
    !p.multi=0
endif
;
; calcul des 2 constantes de temps sur les courbes simulees
;
tt_up  =t_up  (nbp/2-xr/10:nbp/2+2*xr)
tt_down=t_down(nbp/2-xr/10:nbp/2+2*xr)
;
tau_simul_up=AC_TIMECONSTANTE( tt_up, delta_time=delta_t)
tau_simul_down=AC_TIMECONSTANTE(tt_down, delta_time=delta_t)
;
; calcul des 2 constantes de temps theroriques
;
t_100mk=0.1
beta_1=bolo.beta+1
pseudo_pf=bolo.gbol/(beta_1)/(T_100mk^bolo.beta)
;
tmax_up=MAX(tt_up)
tmin_up=MIN(tt_up)
tmax_down=MAX(tt_down)
tmin_down=MIN(tt_down)
;
g_up=pseudo_pf*(tmax_up^beta_1-tmin_up^beta_1)/(tmax_up-tmin_up)
g_down=pseudo_pf*(tmax_down^beta_1-tmin_down^beta_1)/(tmax_down-tmin_down)
tau_theo_up=FLOAT(bolo.c0/g_up)
tau_theo_down=FLOAT(bolo.c0/g_down)
;
; -------
print, 'tau_simul UP:....', tau_simul_up
print, 'tau_theo  UP:....', tau_theo_up
print, '---------------------------------'
print, 'tau_simul DOWN:..', -tau_simul_down
print, 'tau_theo  DOWN:..', tau_theo_down
;
if NOT(KEYWORD_SET(up)) AND NOT(KEYWORD_SET(down)) then up=1
;
if KEYWORD_SET(theory) then begin
    if KEYWORD_SET(up) then return, tau_theo_up
    if KEYWORD_SET(down) then return, tau_theo_down
endif else begin
    if KEYWORD_SET(up) then return, tau_simul_up
    if KEYWORD_SET(down) then return, -tau_simul_down
endelse
;
if KEYWORD_SET(test) then STOP
;
end
