;+
;
; creation par AC le 06/06/2005
;
; Ce programme a pour but de calculer le graphique module du Signal versus
; Frequence pour un signal optique (creneaux) entrants sur les
; bolometres, pour differents courants de polarisation (Ibias)
; 
; Exemple d'appel :
; SIMUL_TIME_RESPONSE, bolo_name='143-1J12', coef=[1.,1.05]*0.8
; SIMUL_TIME_RESPONSE, bolo_name='143-1J12', back=1e-12, sig=5e-14
;
;-
pro SIMUL_TIME_RESPONSE, bolo_name=bolo_name, coefs=coefs, $
                         back=back, signal=signal, $
                         postscript=postscript, test=test, help=help
;
print, 'AC 16/12/2005: This program is OBSOLETE'
print, 'See FitSpeedBolo/ Directory'
return
;
if KEYWORD_SET(help) then begin
    print, 'pro SIMUL_TIME_RESPONSE, bolo_name=bolo_name, coefs=coefs, $'
    print, '                         back=back, signal=signal, $'
    print, '                         postscript=postscript, test=test, help=help'
    print, 'CALLING SEQUENCE:  SIMUL_TIME_RESPONSE, bolo_name=''143-1J12'', coef=[1.,1.05]*0.8'
    return
endif
;
mess=AC_SETMESSAGES('SIMUL_TIME_RESPONSE')
;
AC_RESET
;
; ------------------------------------
; extraction of ONE bolometer in the list
;
if KEYWORD_SET(bolo_name) then begin
    bolo_list=LIST_BOLO_READFILE(/verbose)
    bolo=LIST_BOLO_EXTRACT(bolo_list, name=bolo_name,/verbose)
endif else begin
    bolo=LIST_BOLO_SELECT()
endelse
;
; la structure "bolo" en est-elle vraiment une ?
;
if (N_TAGS(bolo) EQ 0) then begin
    print, mess.fatal+'No Bolo selected'
    return
endif
;
print, mess.info, 'selected bolo is: ', bolo.name
if KEYWORD_SET(verbose) then print, mess.info, 'bolo parameters are:', bolo 
;
; ------------------------------------
; niveau des signaux optiques !
;
ref_lvl=1e-12
low_lvl=0.16e-12
high_lvl=0.35e-12
;
low_lvl=4.e-12
high_lvl=low_lvl+2.e-13
;
if KEYWORD_SET(coefs) then begin
    low_lvl=ref_lvl*coefs(0)
    high_lvl=ref_lvl*coefs(1)
endif else begin
;    stop
    low_lvl=back
    high_lvl=back+signal
endelse
print, low_lvl, high_lvl
;
; liste des courants (Amperes) et nombre de points dans ce tableau
;
courant=[0.02, 0.03, 0.06, 0.12, 0.23, 0.44, 0.82, 1.51, 2.72, 4., 5.]*1e-9
nb_cur=N_ELEMENTS(courant)
;
if (bolo.freq EQ 143.0) then begin
    courant=[0.02, 0.04, 0.07, 0.13, 0.28, 0.58, 1.17, 2.27, 4.32, 8.12]*1e-9
    nb_cur=N_ELEMENTS(courant)   
endif
;
; liste des frequences (Hz) et nombre de points dans ce tableau
;
liste_freqs=[4.,5.,7.,10.,12.,20.,30.,50.,60.,80.,100.,120.,150.,200.,400.]
nb_freqs=N_ELEMENTS(liste_freqs)
;
; nombre de points dans la serie temporelle.
;
nbp=1.e4
nbp=LONG(nbp)
;
fenetre=2L^13
;
; 2 tableaux resultat car 2 methodes
;
resultsFFT=FLTARR(nb_freqs,nb_cur)
resultsMOD=FLTARR(nb_freqs,nb_cur)
;
for i=0, nb_freqs-1 do begin
    largeur=fenetre/(liste_freqs(i))
    print, largeur
    op=CREATE_CRENEAUX(nbp, largeur, low=low_lvl, high=high_lvl)
    for j=0, nb_cur-1 do begin
        temp=BOLO_DIRECT_MODEL(op, bolo=bolo, cur=courant(j))
        ;;plot, temp[4000:5000],/ynozero
        ;;
        sub_temp=temp(nbp-fenetre:*)
        sub_temp=TEMPERATURE2VOLTAGE(sub_temp, courant(j), bolo=bolo)
        FFT_temp=ABS(FFT(sub_temp)) ;;*HANNING(fenetre,alpha=0.56)))
        indice=fenetre/largeur
        resultsFFT(i,j)=SQRT(FFT_temp(indice)^2+FFT_temp(indice-1)^2+FFT_temp(indice+1)^2)
        debug=1
        if (debug EQ 0) and (j EQ 0) then begin
            plot, FFT_temp, xrange=[0,3*indice],/ylog
            ac_lines, indice,/vertical,/ylog, color=!col.g
            print, i, j, indice, FFT_temp(indice)
            ac_delais
        endif
        ;;
        debut=4000
        resultsMOD(i,j)=SQRT(TOTAL( sub_temp^2)/fenetre)
        ;;MAX(temp(debut:*))-MIN(temp(debut:*))
    endfor
endfor
;
; le trace
;
tab_col_ref=[!col.r,!col.g,!col.cb,!col.o,!col.y,!col.b,!col.lg]
tab_col=[tab_col_ref,tab_col_ref]
;
ligne_style_ref=[2,3,4]
ligne_style=[ligne_style_ref,ligne_style_ref,ligne_style_ref,ligne_style_ref]
;
if (bolo.freq EQ 143.0) then begin
    print, mess.info+'custom colors for 143 GHz'
    tab_col=[!col.b,!col.pv,!col.y, !col.g, !col.o, !col.r, !col.pv, !col.b, !col.lg]
    tab_col=[tab_col_ref,tab_col_ref]
endif
;
if KEYWORD_SET(postscript) then begin
    filename='TimeConstFFT_'+bolo.name+'.ps'
    SET_PLOT, 'PS'
    device, file=filename, bits=8, /color
endif else begin
    window,0
endelse
;
cut=AC_MAX2IJ(resultsFFT)
plot, liste_freqs, resultsFFT(*,cut(1)), /xlog,/nodata, $
  title='Done on Fourier, bolo: '+bolo.name, $
  xtitle='Frequency (Hz)', ytitle='signal (abs 1er harmo FFT)'
;
ymax=!y.crange(1)
;
for j=0, nb_cur-1 do begin
    oplot, liste_freqs, resultsFFT(*,j), color=tab_col(j), line=ligne_style(j)
    ;; legende
;    print, ymax, ymax/2.
    y=0.5*ymax+0.4*ymax*(nb_cur-j)/FLOAT(nb_cur)
;    print, y
    plots, [100.,200.], [y,y], color=tab_col(j), line=ligne_style(j)
    XYOUTS, 210., y, courant(j)*1e9
    XYOUTS, 100, .4*ymax, 'Back='+string(back)
    XYOUTS, 100, .45*ymax, 'Signal='+string(signal)
endfor
;
if KEYWORD_SET(postscript) then begin
    device,/close
    set_plot, 'x'
endif
;
if KEYWORD_SET(postscript) then begin
    filename='TimeConstMOD_'+bolo.name+'.ps'
    SET_PLOT, 'PS'
    device, file=filename, bits=8, /color
endif else begin
    window, 1
endelse
;
cut=AC_MAX2IJ(resultsMOD)
plot, liste_freqs, resultsMOD(*,cut(1)), /xlog,/nodata, $
  title='Done on Max-Min, bolo: '+bolo.name, $
  xtitle='Frequency (Hz)', ytitle='signal (abs 1er harmo MOD)'
;
ymax=!y.crange(1)
;
for j=0, nb_cur-1 do begin
    oplot, liste_freqs, resultsMOD(*,j), color=tab_col(j), line=ligne_style(j)
    ;; legende
;    print, ymax, ymax/2.
    y=0.5*ymax+0.4*ymax*(nb_cur-j)/FLOAT(nb_cur)
;    print, y
    plots, [100.,200.], [y,y], color=tab_col(j) , line=ligne_style(j)
    XYOUTS, 210., y, courant(j)*1e9
    XYOUTS, 100, .4*ymax, 'Back='+string(back)
    XYOUTS, 100, .45*ymax, 'Signal='+string(signal)
endfor
if KEYWORD_SET(postscript) then begin
    device,/close
    set_plot, 'x'
endif
;
if KEYWORD_SET(test) then STOP
;
end
